@@ -1,8 +1,8 @@
-/* $Id: FbiStuff.h 116 2008-02-02 13:39:37Z dezperado $ */
+/* $LastChangedDate: 2015-12-19 01:58:26 +0100 (Sat, 19 Dec 2015) $ */
 /*
  FbiStuff.h : Misc fbi functionality routines, modified for fim
 
- (c) 2008 Michele Martone
+ (c) 2008-2015 Michele Martone
  (c) 1998-2006 Gerd Knorr <kraxel@bytesex.org>
 
     This program is free software; you can redistribute it and/or modify
@@ -23,8 +23,8 @@
  * This file comes from fbi, and will undergo severe reorganization.
  * */
 
-#ifndef FBI_STUFF_FBI_H
-#define FBI_STUFF_FBI_H
+#ifndef FIM_FBI_STUFF_H
+#define FIM_FBI_STUFF_H
 
 #include "fim.h"
 
@@ -34,7 +34,7 @@
 
 #ifdef USE_X11
 # include <X11/Intrinsic.h>
-#endif
+#endif /* USE_X11 */
 #include <stdio.h>//FILE
 #include <stdlib.h>//free()
 #include <errno.h>//free()
@@ -55,11 +55,19 @@ namespace fim
 class FbiStuff{
 public:
 static void free_image(struct ida_image *img);
-static struct ida_image* read_image(char *filename);
-static struct ida_image* rotate_image(struct ida_image *src, unsigned int rotation);
-static struct ida_image* scale_image(struct ida_image *src, float scale, float ascale);
-
-static int fim_filereading_debug();
+static FILE* fim_execlp(const fim_char_t *arg, ...);/* new */
+static struct ida_image* read_image(const fim_char_t *filename, FILE* fd, fim_int page=0, Namespace *nsp=FIM_NULL);
+#if FIM_WANT_EXPERIMENTAL_MIPMAPS
+static fim_err_t fim_mipmaps_compute(const struct ida_image *src, fim_mipmap_t * mmp);
+#endif /* FIM_WANT_EXPERIMENTAL_MIPMAPS */
+static struct ida_image* rotate_image90(struct ida_image *src, unsigned int rotation);
+static struct ida_image* rotate_image(struct ida_image *src, float angle);
+static struct ida_image* scale_image(const struct ida_image *src, float scale, float ascale
+#if FIM_WANT_EXPERIMENTAL_MIPMAPS
+		, const fim_mipmap_t * mmp=FIM_NULL
+#endif /* FIM_WANT_EXPERIMENTAL_MIPMAPS */
+	);
+static int fim_filereading_debug(void);
 };
 
 // filter.h
@@ -92,10 +100,10 @@ struct op_rotate_parm {
 /* 
  * dez's function, on the way to windowing Fim!
  * */
-struct ida_image * fbi_image_clone(struct ida_image *img);
+struct ida_image * fbi_image_clone(const struct ida_image *img);
 
 }
 
-#endif
+#endif /* FIM_FBI_STUFF_H */
 
 
