@@ -1,8 +1,8 @@
-/* $Id: FbiStuffFbtools.cpp 114 2008-01-27 13:35:58Z dezperado $ */
+/* $LastChangedDate: 2013-07-04 21:56:00 +0200 (Thu, 04 Jul 2013) $ */
 /*
  FbiStuffFbtools.cpp : fbi functions from fbtools.c, modified for fim
 
- (c) 2008 Michele Martone
+ (c) 2008-2013 Michele Martone
  (c) 1998-2006 Gerd Knorr <kraxel@bytesex.org>
 
     This program is free software; you can redistribute it and/or modify
@@ -28,41 +28,26 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <termios.h>
-#include <signal.h>
-#include <errno.h>
+#include <signal.h>	// sigaction, ...
 #include <setjmp.h>
-#include <sys/ioctl.h>
-#include <sys/mman.h>
-#include <sys/wait.h>
-#include <sys/stat.h>
-
-#include <linux/kd.h>
-#include <linux/vt.h>
-#include <linux/fb.h>
-
-/*#include <asm/page.h>*/ /* seems like this gives problems */
+#ifdef HAVE_SYS_USER_H
 #include <sys/user.h>	  /* this should be a safer replacement */
-#if 0
-#include <signal.h>	  /* added by dez. missing when compiling with -ansi */
-#include <asm/signal.h>	  /* added by dez. missing when compiling with -ansi */
-#endif
+#endif /* HAVE_SYS_USER_H */
 
-//#include "fbtools.h"
 #include "FbiStuffFbtools.h"
 #include "FramebufferDevice.h"
 
 namespace fim
 {
 
-extern FramebufferDevice ffd;
-
 
 
 
 /* -------------------------------------------------------------------- */
 /* handle fatal errors                                                  */
 
-static jmp_buf fb_fatal_cleanup;
+//static jmp_buf fb_fatal_cleanup;	/* old, broken */
+static sigjmp_buf fb_fatal_cleanup;	/* posix */
 
 static void
 fb_catch_exit_signal(int signal)
@@ -75,7 +60,7 @@ void fb_catch_exit_signals(void)
     struct sigaction act,old;
     int termsig;
 
-    memset(&act,0,sizeof(act));
+    fim_bzero(&act,sizeof(act));
     act.sa_handler = fb_catch_exit_signal;
     sigemptyset(&act.sa_mask);
     sigaction(SIGINT, &act,&old);
@@ -92,12 +77,12 @@ void fb_catch_exit_signals(void)
     if (0 == (termsig = sigsetjmp(fb_fatal_cleanup,0)))
 	return;
 
-    /* cleanup */
-    ffd.fb_cleanup();
+    /* console cleanup should happen here */
+    cc.cleanup();
 #ifdef HAVE_SYS_SIGLIST
-    fprintf(stderr,"Oops: %s\n",sys_siglist[termsig]);
-#endif
-    exit(42);
+    FIM_FBI_PRINTF("Oops: %s\n",sys_siglist[termsig]);
+#endif /* HAVE_SYS_SIGLIST */
+    std::exit(FIM_PERR_OOPS);
 }
 
 
